@@ -193,6 +193,7 @@ applet_fill (PanelApplet *applet)
 	gchar *tmp_path;
 	
 	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/drivemount-applet.png");
+	panel_applet_set_flags (applet, PANEL_APPLET_EXPAND_MINOR);
 	
 	panel_applet_add_preferences (applet,
 				      "/schemas/apps/drivemount-applet/prefs",
@@ -395,8 +396,8 @@ browse_cb (BonoboUIComponent *uic,
 		dialog = gtk_message_dialog_new (NULL,
 						 GTK_DIALOG_DESTROY_WITH_PARENT,
 						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
-						 _("There was an error executing '%s' : %s"),
+						 GTK_BUTTONS_OK,
+						 _("There was an error executing '%s': %s"),
 						 command,
 						 error->message);
 
@@ -451,7 +452,7 @@ help_cb (BonoboUIComponent *uic,
 				NULL,
 				GTK_DIALOG_DESTROY_WITH_PARENT,
 				GTK_MESSAGE_ERROR,
-				GTK_BUTTONS_CLOSE,
+				GTK_BUTTONS_OK,
 				_("There was an error displaying help: %s"),
 				error->message);
 
@@ -713,7 +714,7 @@ update_pixmap (DriveData *dd, gint t)
 		text = _(" not mounted");
 	}
 	tiptext = g_strconcat (dd->mount_point, text, NULL);
-	gtk_tooltips_set_tip (dd->tooltips, dd->applet, tiptext, NULL);
+	gtk_tooltips_set_tip (dd->tooltips, dd->button, tiptext, NULL);
 	g_free (tiptext);
 }
 
