@@ -13,28 +13,27 @@
 
 #include <gnome.h>
 #include <panel-applet.h>
-
-
-
-/* Radar map on by default. */
-#define RADARMAP
+#include <libgnomevfs/gnome-vfs.h>
 
 G_BEGIN_DECLS
  
 typedef struct _GWeatherApplet GWeatherApplet;
 typedef struct _GWeatherPrefs GWeatherPrefs;
+typedef struct _WeatherForecast WeatherForecast;
 typedef struct _WeatherInfo WeatherInfo;
 typedef struct _WeatherLocation WeatherLocation;
 
+#define MAX_FORECASTS 5
+
 struct _GWeatherPrefs {
     WeatherLocation *location;
     gint update_interval;  /* in seconds */
     gboolean update_enabled;
     gboolean use_metric;
     gboolean detailed;
-    gboolean radar_enabled;
-    gboolean use_custom_radar_url;
-    gchar *radar;
+    gchar *city;
+    gchar *url;
+    gboolean show_labels;
 };
 
 struct _GWeatherApplet
@@ -45,11 +44,28 @@ struct _GWeatherApplet
 	GtkWidget *box;	
 	GtkWidget *label;
 	GtkWidget *image;
+	GtkWidget *images[MAX_FORECASTS+1];
+    	GtkWidget *boxes[MAX_FORECASTS+1];
+	GtkWidget *labels[MAX_FORECASTS+1];
+	GtkWidget *events[MAX_FORECASTS+1];
 	GtkTooltips *tooltips;
 	
 	PanelAppletOrient orient;
 	gint size;
 	gint timeout_tag;
+	gint animation_tag;
+	gint animation_loc;
+
+	/* Locations stuff */
+	GnomeVFSAsyncHandle *locations_handle;
+	gchar *locations_xml;
+	GtkTreeStore *country_model;
+	GtkWidget *country_tree;
+	GtkTreeStore *city_model;
+	GtkWidget *city_tree;
+	GtkWidget *druid;
+	GtkTreeIter country_iter;
+	gint fetchid; /*-1 not fetching, 0 county, 1 cities */
 	
 	/* preferences  */
 	GWeatherPrefs gweather_pref;
@@ -58,20 +74,9 @@ struct _GWeatherApplet
 
 	GtkWidget *pref_basic_metric_btn;
 	GtkWidget *pref_basic_detailed_btn;
-#ifdef RADARMAP
-	GtkWidget *pref_basic_radar_btn;
-	GtkWidget *pref_basic_radar_url_btn;
-	GtkWidget *pref_basic_radar_url_hbox;
-	GtkWidget *pref_basic_radar_url_entry;
-#endif /* RADARMAP */
 	GtkWidget *pref_basic_update_spin;
 	GtkWidget *pref_basic_update_btn;
-	GtkWidget *pref_net_proxy_auth_btn;
-	GtkWidget *pref_net_proxy_btn;
-	GtkWidget *pref_net_proxy_port_entry;
-	GtkWidget *pref_net_proxy_url_entry;
-	GtkWidget *pref_net_proxy_user_entry;
-	GtkWidget *pref_net_proxy_passwd_entry;
+	GtkWidget *pref_location_city_label;
 	GtkWidget *pref_tree;
 	
 	/* dialog stuff */
@@ -88,15 +93,16 @@ struct _GWeatherApplet
 	GtkWidget *cond_pressure;
 	GtkWidget *cond_vis;
 	GtkWidget *cond_image;
+	GtkWidget *cond_feeltemp;
 	GtkWidget *forecast_text;
 	GtkWidget *radar_image;
+	GtkWidget *forecast_tree;
+	GtkTreeModel *forecast_model;
 
-	GdkPixbuf *dialog_pixbuf;
-	GdkBitmap *dialog_mask;
-	GdkPixbuf *applet_pixbuf;
-	GdkBitmap *applet_mask;
 };
 
+void update_display (GWeatherApplet *applet);
+
 G_END_DECLS
  
 #endif
